class Query:
    """
    As an endpoint to interact with database
    """

    """Queries"""
    __GET_ALL_EXAMPLES = """
        SELECT * FROM EXAMPLE_TABLE
    """

    """GETTERS"""

    @property
    def __GET_ALL_EXAMPLES(self):
        return self.__GET_ALL_EXAMPLES
